# @description Perform hierarchical clustering on a data matrix.
# @param data The data matrix on which to perform the clustering method
# @param row.clustering set to TRUE to perform clustering on the rows, otherwise on columns.
# @param distance.method the distance measure to be used. This must be one of "euclidean", "maximum", "manhattan", "canberra", "binary" or "minkowski". Any unambiguous substring can be given.
# @param p The power of the Minkowski distance.
# @param hclust.method the agglomeration method to be used. This should be (an unambiguous abbreviation of) one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
# @param hclust.members NULL or a vector with length size of data.

matrix.hclust<-function(data,row.clustering=c(TRUE, FALSE), distance.method=c("euclidean","maximum","manhattan", "canberra", "binary","minkowski"), p = 2, hclust.method=c("complete","ward.D", "ward.D2", "single", "average","mcquitty","median","centroid"),hclust.members=NULL)
{
    if (!row.clustering) data<-t(data)

    labels <- rownames(data)

    d<-dist(data, method=distance.method, p)                    # pass p values if minkowski distance is chosen
    hcluster.out<-hclust(d, method=hclust.method, members=hclust.members)
    
    result <- character()
    result_len <- 1
    merges <- hcluster.out$merge
    
    length <- dim(merges)[1]
    
    for (i in seq(1:length)) {
        index1 <- merges[i, 1];
        index2 <- merges[i, 2];
        
        if (index1 < 0) {
            result[result_len] <- paste(labels[-index1], "->", "Cluster ", i, sep = "")
        } else {
            result[result_len] <- paste("Cluster ", index1, "->", "Cluster ", i, sep = "")
        }
        result_len = result_len + 1;
        if (index2 < 0) {
            result[result_len] <- paste(labels[-index2], "->", "Cluster ", i, sep = "")
        } else {
            result[result_len] <- paste("Cluster ", index2, "->", "Cluster ", i, sep = "")
        }
        result_len = result_len + 1;
    }
    
    list(pairs = result, labels = labels)
}