# @description Perform k-means clustering on a data matrix.
# @param data The data matrix on which to perform the clustering method
# @param row.clustering set to TRUE to perform clustering on the rows, otherwise on columns.
# @param centers either the number of clusters, say k, or a set of initial (distinct) cluster centres. If a number, a random set of (distinct) rows in x is chosen as the initial centres.
# @param iter.max the maximum number of iterations allowed.
# @param nstart if centers is a number, how many random sets should be chosen?
# @param algorithm character: may be abbreviated. Note that "Lloyd" and "Forgy" are alternative names for one algorithm.
# @param trace logical or integer number, currently only used in the default method ("Hartigan-Wong"): if positive (or true), tracing information on the progress of the algorithm is produced. Higher values may produce more tracing information.

matrix.kmeans <- function(data, row.clustering = c(TRUE, FALSE), centers = 5, iter.max = 10L, nstart = 1L, algorithm = c("Hartigan-Wong", "Lloyd", "Forgy", "MacQueen"), trace = c(FALSE, TRUE))
{
    if (!row.clustering) {
        data <- t(data)
    }
    
    labels <- rownames(data)
    
    kmeans.out <- kmeans(data, centers, iter.max, nstart, algorithm, trace)    
    cluster <- kmeans.out$cluster
    
    result <- character()
    for (i in seq(1:length(labels))) {
        result[i] <- paste(labels[i], "->", "Cluster ", cluster[i], sep = "")
    }
    
    list(pairs = result, labels = labels)
}